(************************************************************************)
(*                                                                      *)
(*                  Version 1.0a without error checks                   *)
(*                                                                      *)
(************************************************************************)


MODULE LISTsTest;

(************************************************************************)
(*                                                                      *)
(*                          L I S T s T e s t                           *)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(*           Version 1.0a for Sun/4, SunOS 4.1.0, m2c-Compiler          *)
(*     Version 1.0b for IBM-PC/AT, MSDOS 3.30, Taylor Modula-2 3.00     *)
(*     Version 1.0c for IBM-PC/AT, MSDOS 3.30, Fitted Modula-2 2.00     *)
(*                                                                      *)
(*                       (c) Richard Breuer 1991                        *)
(*            e-mail: ricki@rwthi3.informatik.rwth-aachen.de            *)
(*              mail: Brunssumstr. 6, 5100 Aachen, Germany              *)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Distribution:                                                        *)
(*                                                                      *)
(*   I dedicate this  module  to the public domain.  Please do not send *)
(*   any money.  I do not take any warranty for the correctness of this *)
(*   software.  Especially I cannot  be held responsible for any damage *)
(*   that occurs because of the use of this module.                     *)
(*                                                                      *)
(*   Please send all  critics and suggestions to me.  I will try to use *)
(*   them for further releases.                                         *)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Purpose:                                                             *)
(*                                                                      *)
(*   LISTsTest just tests the module LISTs for integrity.  You can find *)
(*   a file 'Results.txt'  in your distribution which contains the out- *)
(*   put of a correct  running test.  Just compare  this with  your own *)
(*   results and report errors to the author.                           *)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Note:								*)
(*                                                                      *)
(*   An explicit test of the procedures LISTClear,  LISTExtractCurrent,	*)
(*   and LISTDeleteElement is not necessary. They are tested implicitly	*)
(*   by calling LISTKill and LISTRemove respectively.			*)
(*                                                                      *)
(************************************************************************)
(*                                                                      *)
(* Revisions:                                                           *)
(*                                                                      *)
(*   08/22/91	Corrected a bug(?) in the TM2 version [BR]		*)
(*   08/21/91   Versions 1.0a,b,c [BR]                                  *)
(*                                                                      *)
(************************************************************************)

  (* Import from all standard modules. Thus they will also be compiled when
     a make is performed *)

  (* the class LISTofTESTclass is created in LISTofTEST with a sorting order
     on the two key components KEY1 and KEY2 *)
  FROM LISTofTEST IMPORT
    LISTofTESTclass;

  FROM LISTofCARDINAL IMPORT
    LISTofCARDINALclass;

  FROM LISTofSHORTCARD IMPORT
    LISTofSHORTCARDclass;

  FROM LISTofSHORTINT IMPORT
    LISTofSHORTINTclass;

  FROM LISTofINTEGER IMPORT
    LISTofINTEGERclass;

  FROM LISTofREAL IMPORT
    LISTofREALclass;

  FROM LISTofWORD IMPORT
    LISTofWORDclass;

  FROM LISTofCHAR IMPORT
    LISTofCHARclass;

  FROM LISTs IMPORT
    LISTType,
    LISTCompareProcType,
    LISTCompareResultType,
    LISTCreateClass,
    LISTCreate,
    LISTSetUndefined,
    LISTIsDefined,
    LISTKill,
    LISTLength,
    LISTCursorPos,
    LISTTop,
    LISTBottom,
    LISTIsEmpty,
    LISTGotoTop,
    LISTGotoBottom,
    LISTGotoNext,
    LISTGotoPrevious,
    LISTGotoElement,
    LISTGotoPos,
    LISTStep,
    LISTCyclicStep,
    LISTClear,
    LISTRemoveElement,
    LISTRemoveCurrent,
    LISTDeleteElement,
    LISTExtractCurrent,
    LISTGetCurrent,
    LISTReplaceCurrent,
    LISTInsertTop,
    LISTInsertBottom,
    LISTInsertBeforeCursor,
    LISTInsertAfterCursor,
    LISTSort,
    LISTInsertSorted,
    LISTRemoveRange,
    LISTExchangeCurrentNext,
    LISTExchangeCurrentPrevious,
    LISTExchangePos,
    LISTMoveCurrentToTop,
    LISTMoveCurrentToBottom,
    LISTMoveElementBeforePos,
    LISTMoveElementAfterPos,
    LISTCopyCurrentToTop,
    LISTCopyCurrentToBottom,
    LISTCopyElementBeforePos,
    LISTCopyElementAfterPos,
    LISTCopy,
    LISTInsertListBeforeCursor,
    LISTInsertListAfterCursor,
    LISTSplitListBeforeCursor,
    LISTSplitListAfterCursor,
    LISTLoop,
    LISTGet,
    LISTInvert,
    LISTPack,
    LISTCheckConsistency;

  FROM SYSTEM IMPORT
    TSIZE;

  FROM TESTdef IMPORT
    TEST;

  FROM SimpleIO IMPORT
    WriteString,
    WriteLn,
    WriteChar,
    WriteCard;
  
  

  VAR
    i: CARDINAL;
    t: TEST;
    s1,s2: LISTType;

  PROCEDURE Show(P: ARRAY OF CHAR; S: LISTType);

  VAR
    i: CARDINAL;
    val: TEST;
    Found: BOOLEAN;
    Store: CARDINAL;

  BEGIN
    IF (LISTIsDefined(S)) THEN
      LISTCheckConsistency(S);
      WriteString("begin "); WriteString(P); WriteLn;
      WriteString("  Num="); WriteCard(LISTLength(S),2);
      WriteString("  Pos="); WriteCard(LISTCursorPos(S),2);
      (* Store current position *)
      Store := LISTCursorPos(S);
      WriteLn;
      LISTLoop(S);
      i := 0;
      REPEAT
        LISTGet(S,val,Found);
        IF (Found) THEN 
          INC(i);
          WriteCard(i,4);
	  (* output a * if cursor is reached *)
          IF (i=Store) THEN
            WriteString(".*(")
	  ELSE
            WriteString(". (")
          END;
          WriteCard(CARDINAL(val.KEY1),2);
          WriteString(",");
          WriteCard(CARDINAL(val.KEY2),2);
          WriteString(',"');
          WriteString(val.CTS);
          WriteString('")');
          WriteLn
        END
      UNTIL (NOT Found);
      IF (LISTIsEmpty(S)) THEN WriteString("  empty"); WriteLn END;
      WriteString("end "); WriteString(P);
      WriteLn;
      (* restore current position *)
      LISTGotoPos(S,Store)
    ELSE
      WriteString("List is undefined"); WriteLn
    END
  END Show;

  PROCEDURE Enter(P: ARRAY OF CHAR);

  BEGIN
    WriteLn;
    WriteString("ENTER ");
    WriteString(P);
    WriteLn;
  END Enter;
  
  PROCEDURE Leave;
  
  BEGIN
    WriteString("EXIT"); 
    WriteLn
  END Leave;

  PROCEDURE TestNavigations;

  VAR
    Found: BOOLEAN;
    
  BEGIN
    Enter("TestNavigations");
    LISTGotoTop(s1);
    WHILE (NOT LISTBottom(s1)) DO LISTGotoNext(s1) END;
    LISTStep(s1,-5);
    Show("5 before bottom",s1);
    LISTCyclicStep(s1,10);
    Show("cyclically 10 further",s1);
    LISTGotoBottom(s1);
    WHILE (NOT LISTTop(s1)) DO LISTGotoPrevious(s1) END;
    LISTStep(s1,5);
    Show("5 after Top",s1);
    LISTCyclicStep(s1,-10);
    Show("cyclically 10 before",s1);
    LISTGotoPos(s1,2);
    Show("at position 2",s1);
    LISTGotoPos(s1,5);
    Show("at position 5",s1);
    LISTGotoPos(s1,18);
    Show("at position 18",s1);
    t.KEY1 := 4;
    t.KEY2 := 4;
    LISTGotoElement(s1,t,Found);
    Show("at element 4",s1);
    Leave;
  END TestNavigations;

  PROCEDURE TestLISTInvert;
  
  BEGIN
    Enter("TestLISTInvert");
    LISTInvert(s1);
    Show("inverted list",s1);
    Leave;
  END TestLISTInvert;

  PROCEDURE TestLISTInsertListAfterCursor;

  VAR
    s: LISTType;

  BEGIN
    Enter("TestLISTInsertListAfterCursor");
    LISTCopy(s1,s);
    Show("Copy of s1",s);
    LISTGotoBottom(s1);
    LISTInsertListAfterCursor(s1,s);
    Show("Copy appended to s1",s1);
    Leave;
  END TestLISTInsertListAfterCursor;

  PROCEDURE TestLISTInsertListBeforeCursor;

  VAR
    s: LISTType;

  BEGIN
    Enter("TestLISTInsertListBeforeCursor");
    LISTCopy(s1,s);
    LISTGotoTop(s1);
    LISTInsertListBeforeCursor(s1,s);
    Show("Another copy prepended to s1",s1);
  END TestLISTInsertListBeforeCursor;

  PROCEDURE TestLISTSort;

  BEGIN
    Enter("TestLISTSort");
    LISTSort(s1,FALSE);
    Show("Reverse sorted list",s1);
    LISTSort(s1,TRUE);
    Show("Sorted list",s1);
    Leave;
  END TestLISTSort;

  PROCEDURE TestLISTPack;

  BEGIN
    Enter("TestLISTPack");
    LISTPack(s1);
    Show("Packed list",s1);
    Leave;
  END TestLISTPack;

  PROCEDURE TestElementMoving;
  
  BEGIN
    Enter("TestElementMoving");
    LISTGotoPos(s1,10);
    LISTMoveCurrentToTop(s1);
    Show("after Move 10 ToTop",s1);
    LISTMoveCurrentToBottom(s1);
    Show("after Move 1 ToBottom",s1);
    LISTMoveElementAfterPos(s1,5,10);
    Show("after Move 5 After 10",s1);
    LISTMoveElementBeforePos(s1,17,3);
    Show("after Move 17 Before 3",s1);
    Leave;
  END TestElementMoving;

  PROCEDURE TestElementCopying;
  
  BEGIN
    Enter("TestElementCopying");
    LISTGotoPos(s1,10);
    LISTCopyCurrentToTop(s1);
    Show("after Copy 10 ToTop",s1);
    LISTCopyCurrentToBottom(s1);
    Show("after Copy 1 ToBottom",s1);
    LISTCopyElementAfterPos(s1,5,10);
    Show("after Copy 5 After 10",s1);
    LISTCopyElementBeforePos(s1,17,3);
    Show("after Copy 17 Before 3",s1);
    Leave;
  END TestElementCopying;

  PROCEDURE TestElementExchanging;
  
  BEGIN
    Enter("TestElementExchanging");
    LISTGotoPos(s1,10);
    LISTExchangeCurrentNext(s1);
    Show("after Exchange 10 Next",s1);
    LISTGotoPos(s1,12);
    LISTExchangeCurrentPrevious(s1);
    Show("after Exchange 12 Previous",s1);
    LISTExchangePos(s1,3,17);
    Show("after Exchange 3 17",s1);
    Leave;
  END TestElementExchanging;

  PROCEDURE TestReplaceAndGet;
  
  VAR
    Found: BOOLEAN;

  BEGIN
    Enter("TestReplaceAndGet");
    LISTGotoPos(s1,12);
    t.KEY1 := 99;
    t.KEY2 := 99;
    t.CTS[0]  := 'U';
    t.CTS[1]  := 'V';
    t.CTS[2]  := 'W';
    t.CTS[3]  := 'X';
    t.CTS[4]  := 0C;
    LISTReplaceCurrent(s1,t);
    Show("after Replace 12 UVWX",s1);
    LISTGetCurrent(s1,t,Found);
    IF (NOT Found)
    OR (t.KEY1#99)
    OR (t.KEY2#99)
    OR (t.CTS[0]#"U")
    OR (t.CTS[1]#"V")
    OR (t.CTS[2]#"W")
    OR (t.CTS[3]#"X")
    OR (t.CTS[4]#0C) THEN
      WriteString("*** Replace/Get-ERROR!");
      WriteLn;
      HALT
    END;
    Leave;
  END TestReplaceAndGet;

  PROCEDURE TestSplitting;

  VAR
    s2: LISTType;
  
  BEGIN
    Enter("TestSplitting");
    LISTGotoPos(s1,10);
    LISTCreate(s2,LISTofTESTclass);
    LISTSplitListAfterCursor(s1,s2);
    Show("First list after SplitAfter 10",s1);
    Show("Second list after SplitAfter 10",s2);
    LISTGotoPos(s1,5);
    LISTSplitListBeforeCursor(s1,s2);
    Show("First list after SplitBefore 5",s1);
    Show("Second list after SplitBefore 5",s2);
    LISTKill(s1);
    LISTCopy(s2,s1);
    LISTKill(s2);
    Leave;
  END TestSplitting;

  PROCEDURE TestDeleting;
  
  BEGIN
    Enter("TestDeleting");
    LISTGotoPos(s1,5);
    LISTRemoveCurrent(s1);
    Show("after Remove 5",s1);
    t.KEY1 := 33;
    t.KEY2 := 33;
    t.CTS[0]  := 'Z';
    t.CTS[1]  := 'Z';
    t.CTS[2]  := 'Z';
    t.CTS[3]  := 'Z';
    t.CTS[4]  := 0C;
    LISTInsertBeforeCursor(s1,t);
    LISTGotoTop(s1);
    Show("after Insert ZZZZ before Cursor & GotoTop",s1);
    LISTRemoveElement(s1,t);
    Show("after Remove ZZZZ",s1);
    LISTRemoveRange(s1,2,LISTLength(s1)-1);
    Show("after RemoveRange 2..bottom-1",s1);
    Leave;
  END TestDeleting;

BEGIN
  (* not necessary (just an example) *)
  LISTSetUndefined(s1);
  LISTSetUndefined(s2);
  Show("Well, just undefined",s1);

  (* create test sets *)
  LISTCreate(s1,LISTofTESTclass);
  FOR i:=1 TO 20 DO 
    t.KEY1 := i;
    t.KEY2 := i;
    t.CTS[0]  := CHR(ORD('A')+i-1);
    t.CTS[1]  := CHR(ORD('A')+i-1);
    t.CTS[2]  := CHR(ORD('A')+i-1);
    t.CTS[3]  := 0C;
    CASE (i MOD 5) OF
      0: LISTInsertBeforeCursor(s1,t);
    | 1: LISTInsertTop(s1,t);
    | 2: LISTInsertBottom(s1,t);
    | 3: LISTInsertAfterCursor(s1,t);
    | 4: LISTInsertSorted(s1,t,FALSE)
    END
  END;
  Show("first set",s1);

  LISTCreate(s2,LISTofTESTclass);
  FOR i:=10 TO 25 DO 
    t.KEY1 := i;
    t.KEY2 := i;
    t.CTS[0]  := CHR(ORD('A')+i-1);
    t.CTS[1]  := CHR(ORD('A')+i-1);
    t.CTS[2]  := CHR(ORD('A')+i-1);
    t.CTS[3]  := 0C;
    CASE (i MOD 5) OF
      0: LISTInsertTop(s2,t);
    | 1: LISTInsertBeforeCursor(s2,t);
    | 2: LISTInsertBottom(s2,t);
    | 3: LISTInsertAfterCursor(s2,t);
    | 4: LISTInsertSorted(s2,t,FALSE)
    END
  END;
  Show("second set",s2);

  TestNavigations;
  TestLISTInvert;
  TestLISTInsertListAfterCursor;
  TestLISTInsertListBeforeCursor;
  TestLISTSort;
  TestLISTPack;
  TestElementMoving;
  TestElementCopying;
  TestElementExchanging;
  TestReplaceAndGet;
  TestSplitting;
  TestDeleting;
        
  LISTKill(s2);
  LISTKill(s1);


END LISTsTest.
